#!/usr/bin/perl -w

#use Acme::Bleach;
use strict;
use warnings;
use DBI;
use Getopt::Long;

my $taxid  = 0;
my $infile = "";
GetOptions( "invalue=i" => \$taxid, "infile=s" => \$infile );

if ( $taxid != 0 ) {
    &findParentTAXID($taxid);
}
elsif ( $infile eq "stdin" ) {
    while (<STDIN>) {
        print STDERR $_;
        chomp;
        $taxid = $_;
        &findParentTAXID($_);
    }
}
elsif ( $infile ne "" ) {
    die "Can't open the input file $infile\n" unless open( IN, $infile );
    while (<IN>) {
        print STDERR $_;
        chomp;
        $taxid = $_;
        &findParentTAXID($_);
    }
    close(IN);
}
else {
    print STDERR "
#usage: perl findLineage.PL -invalue {input value, e.g. 7}
#usage: perl findLineage.PL -infile {input file name}
#usage: perl findLineage.PL -infile stdin
\n";

}

###################################

sub findParentTAXID {

    my ($currentID) = @_;

    die "Input must be a tax id, which is an interger!\n" if ( $currentID !~ /^\d+$/ );

    my $db_handle =
      DBI->connect(
        "dbi:mysql:database=ITSDB;host=localhost:3306;user=itsdb_user;password=itsdb_pwd")
      or die "Couldn't connect to database: $DBI::errstr\n";

    while ( $currentID != 1 ) {

       #my $sql       = "SELECT * FROM Taxonomy_Nodes where TaxID = $currentID";
        my $sql =
"select Taxonomy_Nodes.*, Taxonomy_Name.* from Taxonomy_Nodes, Taxonomy_Name where Taxonomy_Nodes.TaxID = Taxonomy_Name.TaxID and Taxonomy_Name.NameClass = 'scientific name'and Taxonomy_Name.TaxID = $currentID";
        my $statement = $db_handle->prepare($sql)
          or die "Couldn't prepare query '$sql': $DBI::errstr\n";

        $statement->execute()
          or die "Couldn't execute query '$sql': $DBI::errstr\n";

        #$currentID = 1;    # no sql return value, $currentID will be 1
        while ( my $row_ref = $statement->fetchrow_hashref() ) {
            print

   #  " $taxid\t$row_ref->{TaxID}\t$row_ref->{Rank}\t$row_ref->{ParentTaxID}\n";
"$taxid\t$row_ref->{ParentTaxID}\t$row_ref->{TaxID}\t$row_ref->{Rank}\t$row_ref->{Name}\n";
            $currentID = $row_ref->{ParentTaxID};
        }
    }

    $db_handle->disconnect();
}
