#!/usr/bin/perl

#take in a list of taxids and two input files, the list of OTU taxid, the list of taxid2acc return the list of accession numbers corresponding the the input taxids

=head
sample usage: 

[cliu@ginkgo pt]$ pwd
/home/cliu/pt
[cliu@ginkgo pt]$ cut -f1 byclass | head -2 | perl scripts/pt13_getAccforOTU.pl 
./m2.taxid2acc
./2300lineage
113507   FJ572503 FJ572504
113508   AY309617 GQ428034 AY309607 AY309619 FJ572571 FJ572572
[cliu@ginkgo pt]$ echo 113507 | perl scripts/pt13_getAccforOTU.pl ./m2.taxid2acc
./2300lineage
113507   FJ572503 FJ572504
=cut

use strict;

my @taxids = <STDIN>;

my $infile_taxid2acc = shift;  #|| "./m2.taxid2acc";
my $infile_lineage = shift; # || "./2300lineage";

die "Usage: cat listoftaxid | $0 infile_taxid2acc infile_lineage\n" unless (-e $infile_taxid2acc && -e $infile_lineage); 

my @lines_taxid2acc = `cat $infile_taxid2acc`;
my @lines_lineage = `cat $infile_lineage`;

my $hash_taxid2acc = {};
my $hash_lineage = {};
my $hash_species_ids = {};
my $hash_species_count = {};
my $hash_acc_count = {};
my $hash_otu2acc = {};

#this is to build the hash species taxid->accession 
foreach my $line (@lines_taxid2acc) {
	next if ($line =~ /^1-1/); #here comes the first confusion, the input already are the "1-n"
	$line =~ s/\s+$//g;
	my @ws = split (/\t/, $line);
	$hash_taxid2acc->{$ws[1]} = $ws[2]; #$ws[0] is the 1-n, $ws[1] is the taxid, $ws[2] is the list of accession numbers
}

#this is get the species taxids for all taxids and then concatenate the accession numbers. Save it to hash hash_lineage.
foreach my $line (@lines_lineage) {
        $line =~ s/\s+$//g;
        my @ws = split (/\t/, $line);
	#ws[2] is the OTU taxid ws[0] is the species taxid

	#add the accession to the list only if there are accs for a given species taxid
	if ($hash_taxid2acc->{$ws[0]} ne "") { #keep the species id that has at least one sequence
        	$hash_lineage->{$ws[2]} = $hash_lineage->{$ws[2]}." ".$hash_taxid2acc->{$ws[0]};
		$hash_species_ids->{$ws[2]} =  $hash_species_ids->{$ws[2]}." ".$ws[0];
		my @num_of_accs = split (/\s/, $hash_taxid2acc->{$ws[0]});
		$hash_acc_count->{$ws[2]} = $hash_acc_count->{$ws[2]}+$#num_of_accs+1;
		$hash_species_count->{$ws[2]}++; #count number of species
	}
}

#here print out the accession numbers for a given taxid
foreach my $taxid (@taxids) {
	$taxid =~ s/\s+//;
	print STDERR $taxid,"\t",$hash_species_ids->{$taxid},"\t",$hash_species_count->{$taxid},"\t",$hash_acc_count->{$taxid},"\n";
	print "$taxid\t$hash_lineage->{$taxid}\n" if ($hash_species_count->{$taxid} >=2); #only print taxids that have more than two species	
}
