#!/usr/bin/perl

use strict;
use Time::HiRes qw/gettimeofday/;
use FindBin;
require "$FindBin::Bin/lib_master.pl";
use Getopt::Long;

my $methodType = "";
my $includeself = "";
my $saveTmpFile = "";
my $dist_type = "";
my $jobid = "";
my $file_seq = "";
my $blastdb = "";
my $kmerdb = "";

GetOptions( 
	"methodType=i" => \$methodType, 
	"includeSelf!" => \$includeself, 
	"saveTmpFile!" => \$saveTmpFile,
	"distType:i" => \$dist_type, #the following four options are optional 
	"jobid:s" => \$jobid,
	"sequences:s" => \$file_seq,
	"blastdb:s" => \$blastdb,
	"kmerdb:s" => \$kmerdb,
); 

if ($methodType eq "" || !(-e $file_seq) || !(-e $blastdb)) {
	die "Usage: cat list_of_acc | $0 -m 0-8 -sequence sequence_file -blastdb blastdb -jobid XX -[no]includeSelf
	The following parameters are optional:
	-[no]saveTmpFile
	-distType 
	-sequence 
	-blastdb
	-kmerdb
	-jobid

	The method type can be 0-7, the corresponding algorithm types are:

	Method type: Algorithm type
	0: 0
	1: 0+1
	2: 0+2
	3: 3
	4: 3+1
	5: 3+2
	6: 0+1+2
	7: 3+1+2
	8: 1

	Algorithm type:
	0: Blast
	1: P Distance
	2: Edit Distance
	3: Kmer

";
}

##Set up environment variables
#f ($blastdb eq "") { $blastdb = "$FindBin::Bin/../databases/m2.11137"; }
if ($kmerdb eq "") { $kmerdb = "$FindBin::Bin/../databases/kmerdb.11137"; }
#if ($file_seq eq "") { $file_seq = "$FindBin::Bin/../databases/seq_1line_11137"; }
if ($includeself eq "") { $includeself = 1; }
if ($saveTmpFile eq "") { $saveTmpFile = 0; }

print STDERR "You entered: method_type:$methodType, include_self:$includeself, save_tmp_file:$saveTmpFile\n";

open (IN, $file_seq) || die "failed to open file $file_seq\n";

if ($dist_type eq "") {
	$dist_type = 0;
}
print STDERR "$blastdb\n";

my $filebase = "";
my $count = 0;
my $href_seqs = {};
my $href_taxid = {};
my $href_hits = {};

my $correctlyIdentified = "";
my $list_of_correctly_identified = "";
my $list_of_wrongly_identified = "";

my @lines = <IN>;
close (IN);
($href_seqs, $href_taxid) = &loadseq (\@lines);

##Step 1, pre-filtering step, do the blast and get the best hits
my @query_ids = <STDIN>;
chomp(@query_ids);

my $start_time = gettimeofday();

foreach my $query_id (@query_ids) { #start the loop, process one $query_id at a time

        $query_id =~ s/\s+$//g;
        print STDERR "$count\t$query_id\n";
        $count++;
        my $time = gettimeofday();
        $time =~ s/\.//;
        $filebase = "/share/work/tmp/job".$jobid."_".$time;

	###############Blast method alone m0
	if ($methodType == 0) {

                my $file_blast = &doblast($filebase, $blastdb, $href_seqs, $query_id);
                $href_hits = &parseBlastFile($file_blast, $includeself); #query_id, list of hits, can do a batch here
		$correctlyIdentified = &identifybyBlast($href_taxid, $query_id, $href_hits, $dist_type);
	} 

	###############Blast+P Distance method m0+1
	elsif ($methodType == 1) {

	        my $file_blast = &doblast($filebase, $blastdb, $href_seqs, $query_id);
        	$href_hits = &parseBlastFile($file_blast, $includeself); #query_id, list of hits, can do a batch here

        	##Step 2, identify based on different distance based methods 
        	foreach my $hit_id (keys %$href_hits) {
			print "$hit_id\t$href_hits->{$hit_id}\n";
        	}

		my $href_p_hits = {};
                ($correctlyIdentified, $href_p_hits) = &identifybyDistance($filebase, $href_seqs, $href_taxid, $href_hits, $query_id, $dist_type, $includeself, $saveTmpFile);
		#print "----$correctlyIdentified-----------\n";
	}

	###############Blast+Edit Distance method m0+2
	elsif ($methodType == 2) {

	        my $file_blast = &doblast($filebase, $blastdb, $href_seqs, $query_id);
        	$href_hits = &parseBlastFile($file_blast, $includeself); #query_id, list of hits, can do a batch here

        	foreach my $hit_id (keys %$href_hits) {
                	print "$hit_id\t$href_hits->{$hit_id}\n";
		}
                $correctlyIdentified = &identifybyLD($filebase, $href_seqs, $href_taxid, $href_hits, $query_id, $dist_type, $includeself, $saveTmpFile);
        	
	} 

	###############kmer method alone 
	elsif ($methodType == 3) {

		$href_hits = &doKmerAnalysis($filebase, $href_seqs, $kmerdb, $query_id, $includeself);
                $correctlyIdentified=&identifybyBlast($href_taxid, $query_id, $href_hits, $dist_type);
	}
	
	###############kmer+P distance method
	elsif ($methodType == 4) { #kmer+P Distance 3+1

                $href_hits = &doKmerAnalysis($filebase, $href_seqs, $kmerdb, $query_id, $includeself);
                $correctlyIdentified=&identifybyBlast($href_taxid, $query_id, $href_hits, $dist_type);

		if ($correctlyIdentified == 1) { #the end
		} else { #move to the next method

		        my $file_blast = &doblast($filebase, $blastdb, $href_seqs, $query_id);
	                $href_hits = &parseBlastFile($file_blast, $includeself); #query_id, list of hits, can do a batch here

                	##Step 2, identify based on different distance based methods 
                	foreach my $hit_id (keys %$href_hits) {
                        	print "$hit_id\t$href_hits->{$hit_id}\n";
                	}
		my $href_p_hits = {};
                ($correctlyIdentified, $href_p_hits)= &identifybyDistance($filebase, $href_seqs, $href_taxid, $href_hits, $query_id, $dist_type, $includeself, $saveTmpFile);

		}
	}	

	###############kmer+Edit distance method ######
        elsif ($methodType == 5) { #kmer+Edit Distance 3+1

                $href_hits = &doKmerAnalysis($filebase, $href_seqs, $kmerdb, $query_id, $includeself);
                $correctlyIdentified=&identifybyBlast($href_taxid, $query_id, $href_hits, $dist_type);

                if ($correctlyIdentified == 1) { #the end
                } else { #move to the next method

                        my $file_blast = &doblast($filebase, $blastdb, $href_seqs, $query_id);
                        $href_hits = &parseBlastFile($file_blast, $includeself); #query_id, list of hits, can do a batch here

                        ##Step 2, identify based on different distance based methods 
                        foreach my $hit_id (keys %$href_hits) {
                                print "$hit_id\t$href_hits->{$hit_id}\n";
                        }
   			$correctlyIdentified = &identifybyLD($filebase, $href_seqs, $href_taxid, $href_hits, $query_id, $dist_type, $includeself, $saveTmpFile);

                }
        }

 	###################Blast+P Distance+Edit Distance
	elsif ($methodType == 6) { 

                my $file_blast = &doblast($filebase, $blastdb, $href_seqs, $query_id);
                $href_hits = &parseBlastFile($file_blast, $includeself); #query_id, list of hits, can do a batch here
                $correctlyIdentified = &identifybyBlast($href_taxid, $query_id, $href_hits, $dist_type);

                if ($correctlyIdentified == 1) { #the end
                } else { #move to the next method

                        ##Step 2, identify based on different distance based methods 
                        foreach my $hit_id (keys %$href_hits) {
                                print "$hit_id\t$href_hits->{$hit_id}\n";
                        }
			
			my $href_p_hits = {};
                        ($correctlyIdentified, $href_p_hits) = &identifybyDistance($filebase, $href_seqs, $href_taxid, $href_hits, $query_id, $dist_type, $includeself, $saveTmpFile);

                        #loop for edit distance
                        if ($correctlyIdentified == 1) { #the end
                         } else { #move to the next method

                                ##Step 2, identify based on different distance based methods 
                                foreach my $hit_id (keys %$href_p_hits) {
                                        print "$hit_id\t$href_p_hits->{$hit_id}\n";
                                }
                                $correctlyIdentified = &identifybyLD($filebase, $href_seqs, $href_taxid, $href_p_hits, $query_id, $dist_type, $includeself, $saveTmpFile);

                        }
                }
	}

	#####################Kmer+P Distance+Edit Distance
	elsif ($methodType == 7) { 

                $href_hits = &doKmerAnalysis($filebase, $href_seqs, $kmerdb, $query_id, $includeself);
                $correctlyIdentified=&identifybyBlast($href_taxid, $query_id, $href_hits, $dist_type);

                if ($correctlyIdentified == 1) { #the end
                } else { #move to the next method

                        my $file_blast = &doblast($filebase, $blastdb, $href_seqs, $query_id);
                        $href_hits = &parseBlastFile($file_blast, $includeself); 

                        ##Step 2, identify based on different distance based methods 
                        foreach my $hit_id (keys %$href_hits) {
                                print "$hit_id\t$href_hits->{$hit_id}\n";
                        }
                        ($correctlyIdentified, $href_hits) = &identifybyDistance($filebase, $href_seqs, $href_taxid, $href_hits, $query_id, $dist_type, $includeself, $saveTmpFile);

			#loop for edit distance
	                if ($correctlyIdentified == 1) { #the end
       		         } else { #move to the next method

                    	    	##Step 2, identify based on different distance based methods 
                        	foreach my $hit_id (keys %$href_hits) {
                                	print "$hit_id\t$href_hits->{$hit_id}\n";
                      		}
                  	      	$correctlyIdentified = &identifybyLD($filebase, $href_seqs, $href_taxid, $href_hits, $query_id, $dist_type, $includeself, $saveTmpFile);

                	}
                }
	}

        ###############P Distance method m1
        elsif ($methodType == 8) { #only test the P distance

                #my $file_blast = &doblast($filebase, $blastdb, $href_seqs, $query_id);
                #$href_hits = &parseBlastFile($file_blast, $includeself); #The key is query_id, and the value is list of hits, can do a batch here
		#how to do the p distance method without the blast filtering
		my $query_ids = join(" ", @query_ids);
		$href_hits->{$query_id} = $query_ids;
		print "$query_ids\n";

                ##Step 2, identify based on different distance based methods 
                foreach my $hit_id (keys %$href_hits) {
                        #print "$hit_id\t$href_hits->{$hit_id}\n";
                }

                my $href_p_hits = {};
                ($correctlyIdentified, $href_p_hits) = &identifybyDistance($filebase, $href_seqs, $href_taxid, $href_hits, $query_id, $dist_type, $includeself, $saveTmpFile);
                #print "----$correctlyIdentified-----------\n";
        }

	print "result: $correctlyIdentified\n";
        if ($correctlyIdentified == 1) {
		print "method$methodType\tcorrect\t".$query_id."\n";
	} elsif ($correctlyIdentified == -1) {
		print "method$methodType\tfailed\t".$query_id."\n";
	} else {
		print "method$methodType\twrong\t".$query_id."\n";
	}

	###############clean up the tmp file
        if ($saveTmpFile eq "0") {
                my $intermediate_files = $filebase."*";
		`rm $intermediate_files`;
                print STDERR "rm $intermediate_files\n";
        }
}

my $end_time = gettimeofday();
my $no_of_ids = $#query_ids+1;

print "total time run: ", $end_time-$start_time, " seconds for ", $no_of_ids, " queries.\n";

