#!/usr/bin/perl

use strict; 
use Time::HiRes qw/gettimeofday/;

##############################################################################

sub doKmerAnalysis {
	my ($filebase, $href_seqs, $kmerdb, $query_id, $includeself) = @_;
	my $file_kmer_in = $filebase.".kmerin";
	my $file_kmer_out = $filebase.".kmerout";
	open (IN, $kmerdb);
	my @lines = <IN>;
	my $noInput = ($#lines+1)/2;
	my $query_seq = $href_seqs->{$query_id};
	`echo $query_seq > $file_kmer_in`;
	`/share/apps/bin/identifybykmer_test $file_kmer_in $kmerdb $file_kmer_out $noInput $includeself $query_id`;

	my $href_hits = {};
	open (IN, $file_kmer_out) || die "Can't open $file_kmer_out\n";
	my @lines = <IN>;
	foreach my $line (@lines) {
		my ($query_id, $hits) = split (/\t/, $line);
		if ($includeself) { $hits = $query_id." ".$hits; }
		$href_hits->{$query_id} = $hits;
	}
	
	return $href_hits; 
}


###############################################################################

sub identifybyBlast {

	my ($href_taxid, $query_id, $href_hits, $dist_type) = @_;
	$dist_type = "blast";
	my $list_of_acc = $href_hits->{$query_id};
        my @minaccs = split (/\s+/, $list_of_acc);

        print "t$dist_type\ta2a\t$query_id\t@minaccs\n";
        my @taxids = &accs2taxids($href_taxid, $list_of_acc);
        print "t$dist_type\ta2t\t$query_id\t$href_taxid->{$query_id}\t@taxids\n";

	my $isCorrectIdentification = &isCorrectIdentification ($href_taxid->{$query_id}, @taxids);
	return $isCorrectIdentification;
}

###############################################################################

sub identifybyKmer {

        my ($href_taxid, $query_id, $href_hits, $dist_type) = @_;
	$dist_type = "kmer";
        my $list_of_acc = $href_hits->{$query_id};
        my @minaccs = split (/\s+/, $list_of_acc);

        print "t$dist_type\ta2a\t$query_id\t@minaccs\n";
        my @taxids = &accs2taxids($href_taxid, $list_of_acc);
        print "t$dist_type\ta2t\t$query_id\t$href_taxid->{$query_id}\t@taxids\n";

        my $isCorrectIdentification = &isCorrectIdentification ($href_taxid->{$query_id}, @taxids);
        return $isCorrectIdentification;
}

##############################################################################

sub isCorrectIdentification {
	my ($query_taxid, @hit_taxids) = @_;
	#print STDERR "$query_taxid\t@hit_taxids\n";
	my $result = "";

	if ( $#hit_taxids < 0 ) {
		$result = -1;
	} elsif ( $#hit_taxids == 0 && $query_taxid eq $hit_taxids[0]) {
		$result = 1;
	} elsif ($#hit_taxids > 0) {
		$result = 0;
	} else {
		$result = 2;
	} 

	return $result;
}

###############################################################################
sub identifybyDistance {

	my ($filebase, $href_seqs, $href_taxid, $href_hits, $query_key, $dist_type, $includeself, $saveTmpFiles) = @_;
	my $query = "";
	my $href_results = {};
        my @accs = split (/\s+/, $href_hits->{$query_key});
	my @accs = uniq(@accs);

	print STDERR "Retrieve accession numbers: ---$href_hits->{$query_key}--- @accs for $query_key\n";

	foreach my $acc (@accs) {

		if ($query_key eq $acc) {
			if ($includeself eq "1") {
				$href_results->{$acc}="0.00";
			} 
			next;
		} #not include self. 

		my $postfix = $query_key."_".$acc;
        	my $file_fas = $filebase."_".$postfix.".fas";
        	my $file_aln = $filebase."_".$postfix.".aln";
       		my $file_distmat = $filebase."_".$postfix.".distmat";

        	open (OUT, ">>$file_fas") || die "failed to open $file_fas to write\n";
		print OUT ">ref_$query_key\n";
		print OUT "$href_seqs->{$query_key}\n";
        	print OUT ">ref_$acc\n";
	        print OUT "$href_seqs->{$acc}\n";
	        close (OUT);

		print STDERR "clustalw\n";
	        `/share/apps/bin/clustalw -infile=$file_fas -outfile=$file_aln`; #do multiple sequence alignment
		print STDERR "distmat\n";
	        `/share/apps/bin/distmat -sequence $file_aln -nuc $dist_type -out $file_distmat`; #calculate distance matrix
        	my ($query, $hit_acc, $dis) = &parseDistmatMatrixfor2Seq($file_distmat);
		$href_results->{$hit_acc}=$dis;

	}

	foreach my $key (keys %$href_results) {
		print "t0\t121\t$query_key\t$key\t$href_results->{$key}\n";
	}

	my $list_of_acc = &getKeyswithMiniValue($href_results);
	my @minaccs = split (/\s+/, $list_of_acc);
        print "t$dist_type\ta2a\t$query_key\t@minaccs\n";
        my @taxids = &accs2taxids($href_taxid, $list_of_acc);
        print "t$dist_type\ta2t\t$query_key\t$href_taxid->{$query_key}\t@taxids\n";

        my $isCorrectIdentification = &isCorrectIdentification ($href_taxid->{$query_key}, @taxids);
        return $isCorrectIdentification, &array2hash($query_key, @minaccs) ;

}

############################

sub array2hash {
	my ($key, @array) = @_;
	my $href = {};

	my $hits = join " ", @array;
	$href->{$key} = $hits;
	return $href;
}

#########################################################################
sub identifybyLD {

	my ($filebase, $href_seqs, $href_taxid, $href_hits, $hit_id, $dist_type, $includeself, $saveTmpFile) = @_;
	my @accs = split (/\s+/, $href_hits->{$hit_id});
	my @accs = uniq(@accs);

	$dist_type ='ld';
	print STDERR "calcualte ld for $hit_id\n";
        #calculate alignement_free distance
        my $query_seq = $href_seqs->{$hit_id};
        my $href_results = {};
        foreach my $acc (@accs) {
                my $target_seq = $href_seqs->{$acc};
                my $ld=&levenshtein($query_seq, $target_seq);
                $href_results->{$acc} = $ld;
		print "t$dist_type\ta2a_ld\t$hit_id\t$acc\t$ld\n";

        }

        #calculate alignement_free distance
        my $list_of_acc = &getKeyswithMiniValue($href_results);
	my @minaccs = split (/\s+/, $list_of_acc);
        print "t$dist_type\ta2a\t$hit_id\t@minaccs\n";
        my @taxids = &accs2taxids($href_taxid, $list_of_acc);
        print "t$dist_type\ta2t\t$hit_id\t$href_taxid->{$hit_id}\t@taxids\n";

        my $isCorrectIdentification = &isCorrectIdentification ($href_taxid->{$hit_id}, @taxids);
        return $isCorrectIdentification;

}
#########################################################################
sub uniq {
    return keys %{{ map { $_ => 1 } @_ }};
}

#########################################################################
sub accs2taxids {

	my ($href_taxid, $list_of_acc) = @_;
	my @accs = split(/\s+/, $list_of_acc);
	my @taxids = ();
	foreach my $acc (@accs) {
		$acc =~ s/ref_//;
		push @taxids, $href_taxid->{$acc};
	}
	return uniq(@taxids);
}

#########################################################################
sub getKeyswithMiniValue {

	my ($href) = @_;
	my ($result, $min) = "";

	for my $key (keys %$href) {
		if ($min eq "") {
                        $min = $href->{$key};
                } elsif ($min > $href->{$key}) {
                        $min = $href->{$key};
		}
	}
	
	for my $key (keys %$href) {
		if ($min == $href->{$key}) {
			$result .= $key."\t";
		} else {

		} 	
        }

	return $result;
}

##########################################################################
sub parseDistmatMatrixfor2Seq {

        my ($file_distmat) = @_;

        open (IN, $file_distmat) || die "Failed to open file\n";
        my @lines = (<IN>);
        my @parts = split (/_/, $file_distmat); #parse the file name to get the key
        my $hit_acc = pop @parts;
	my $query = pop @parts;
        $hit_acc =~ s/\.distmat//;

	my @tmp = grep {/$query/} @lines; #get all keys
	my $line = $tmp[0];
	my @eles = split (/\s+/, $line);
	pop(@eles);
	pop(@eles);
	my $dis=pop(@eles);	
	return ($query, $hit_acc, $dis);
}

###########################################################################3
sub parseDistmatMatrix {

	my ($file_distmat) = @_;

	open (IN, $file_distmat) || die "Failed to open file\n";
	my @lines = (<IN>);
	
	my @parts = split (/_/, $file_distmat); #parse the file name to get the key
	my $key = pop @parts;
	$key =~ s/\.distmat//;
	#print "$key\n";
        my @row = grep {/$key/} @lines; #determine the order of the key
        my @tmps = split (/\s+/, $row[0]);
        my $order = pop(@tmps);
        #print @row, "\n", $order, "\n";

	my @ref_lines = grep {/ref_/} @lines; #get all keys
	#print STDERR @ref_lines, "\n";
	
	#build the 2d array
	my $href_table = {};
	my $href_name = {};
	my ($i, $rank) = "";

	foreach my $line (@ref_lines) {
		$line =~ s/^\s+//;
		my @eles = split (/\s+/, $line);
		$rank = pop(@eles);
		my $name = pop(@eles);
		my $href_row = {};
		for (my $i=0; $i<=$#eles; $i++)  {
			$href_row->{$i+$rank} = $eles[$i];
		}
		$href_name->{$rank} = $name;
		$href_table->{$rank} = $href_row;
	}

	#select the values
	my $href_results = {};
	for ($i=1; $i<=$order; $i++) {
		$href_name->{$i} =~ s/ref_//g;
		$href_results->{$href_name->{$i}} = $href_table->{$i}->{$order};
	}

	for ($i=$order+1; $i<=$rank; $i++) {
		$href_name->{$i} =~ s/ref_//g;
		$href_results->{$href_name->{$i}} = $href_table->{$order}->{$i};
        }

	return $key, $href_results;

}

####################################################################
sub loadseq {
	my ($aref) = @_;
	my $href_seqs = {}; #key is acc, value is the sequence
	my $href_taxid = {}; #key is acc, value is the taxid

	foreach my $line (@$aref) {
		$line =~ s/\s+$//;
		my @tmps = split(/_/, $line);
		$href_seqs->{$tmps[2]} = $tmps[3];
		$href_taxid->{$tmps[2]} = $tmps[1];
	}
	
	return $href_seqs, $href_taxid;
}

#####################################################################
sub to1line {

	my (@lines) = @_;
	my $i = 0; 
	my $seq = "";
	foreach my $line (@lines) {
		$line =~ s/\s+$//;
		print STDERR "$i\n"; $i++;
		if ($line =~ /^>/) {
			$seq = $seq.$line."_";
		} else {
			$seq = $seq.$line."\n";
		}	
	}
	return $seq;
}


#####################################################################
sub doblast {
        my ($filebase, $blastdb, $href_seqs, $query_id) = @_;
        my $file_query=$filebase.".query";
        my $file_fasta=$filebase.".fa";
        my $file_blast=$filebase.".blast";

        open (OUT, ">$file_query");
        print OUT ">$query_id\n",$href_seqs->{$query_id};
        close (OUT);
	print STDERR "$file_query\t$blastdb\t$file_blast\n";
        `/share/apps/bin/blastall -p blastn -i $file_query -d $blastdb -m 8 -e 1e-10 -o $file_blast`;
        #`/share/apps/bin/blastall -p blastn -i $file_query -d $blastdb -m 8 -F F -e 1e-10 -o $file_blast`;
	return $file_blast;
}

###################################################
sub parseBlastFile {

        my ($file_blast, $includeself) = @_;
        open (IN, $file_blast);
        my (@lines) = <IN>;
        close(IN);
        chomp(@lines);
        my $result = "";
        my $href_max_score = {};
	my $href_hits = {};

	#find the max score
	foreach my $line (@lines) {
                $line =~ s/refbc_//g; #delete refbc
                $line =~ s/\s+$//g;
                my (@ws) = split (/\s+/, $line);

                my $query_acc = $ws[0]; #query only contain acc
                my ($hit_taxid, $hit_acc) = split(/_/, $ws[1]); #subject contain acc+taxid

                if ($includeself eq "0" && $query_acc eq $hit_acc) {
			next;
		}

                if ($href_max_score->{$query_acc} eq "") {
                        $href_max_score->{$query_acc} = $ws[11];
                } else { #when $href_max->{$ws[0]}  ne ""
                        if (int($ws[11]) >= int($href_max_score->{$query_acc})) {
                                $href_max_score->{$query_acc} = $ws[11];
			}
                }
        }

        foreach my $line (@lines) {
                $line =~ s/refbc_//g; #delete refbc
                $line =~ s/\s+$//g;
                my (@ws) = split (/\s+/, $line);

                my $query_acc = $ws[0]; #query only contain acc
                my ($hit_taxid, $hit_acc) = split(/_/, $ws[1]); #subject contain acc+taxid

                if ($includeself eq "0" && $query_acc eq $hit_acc) {
                        next;
                }

                if ($ws[11] eq $href_max_score->{$query_acc} ) {
                        $href_hits->{$query_acc} .= $hit_acc." ";
                } 
        }

        return $href_hits;
}

##############################################################################
# Levenshtein, who devised the algorithm in 1965
sub levenshtein
{
    # $s1 and $s2 are the two strings
    # $len1 and $len2 are their respective lengths
    #
    my ($s1, $s2) = @_;
    my ($len1, $len2) = (length $s1, length $s2);

    # If one of the strings is empty, the distance is the length
    # of the other string
    #
    return $len2 if ($len1 == 0);
    return $len1 if ($len2 == 0);

    my %mat;

    # Init the distance matrix
    #
    # The first row to 0..$len1
    # The first column to 0..$len2
    # The rest to 0
    #
    # The first row and column are initialized so to denote distance
    # from the empty string
    #
    for (my $i = 0; $i <= $len1; ++$i)
    {
        for (my $j = 0; $j <= $len2; ++$j)
        {
            $mat{$i}{$j} = 0;
            $mat{0}{$j} = $j;
        }

        $mat{$i}{0} = $i;
    }

    # Some char-by-char processing is ahead, so prepare
    # array of chars from the strings
    #
    my @ar1 = split(//, $s1);
    my @ar2 = split(//, $s2);

    for (my $i = 1; $i <= $len1; ++$i)
    {
        for (my $j = 1; $j <= $len2; ++$j)
        {
            # Set the cost to 1 iff the ith char of $s1
            # equals the jth of $s2
            # 
            # Denotes a substitution cost. When the char are equal
            # there is no need to substitute, so the cost is 0
            #
            my $cost = ($ar1[$i-1] eq $ar2[$j-1]) ? 0 : 1;

            # Cell $mat{$i}{$j} equals the minimum of:
            #
            # - The cell immediately above plus 1
            # - The cell immediately to the left plus 1
            # - The cell diagonally above and to the left plus the cost
            #
            # We can either insert a new char, delete a char or
            # substitute an existing char (with an associated cost)
            #
            $mat{$i}{$j} = min([$mat{$i-1}{$j} + 1,
                                $mat{$i}{$j-1} + 1,
                                $mat{$i-1}{$j-1} + $cost]);
        }
    }

    # Finally, the Levenshtein distance equals the rightmost bottom cell
    # of the matrix
    #
    # Note that $mat{$x}{$y} denotes the distance between the substrings
    # 1..$x and 1..$y
    #
    return $mat{$len1}{$len2};
}

# minimal element of a list
#
sub min
{
    my @list = @{$_[0]};
    my $min = $list[0];

    foreach my $i (@list)
    {
        $min = $i if ($i < $min);
    }

    return $min;
}

1;
